"""
Save regular expressions.
"""
import re

META_PATTERN = re.compile(
rb'''(?P<META_HEADER>MARKS_GROUPS_DATA; BEGIN; COMPOSITE;BINARY;\d+
VERSION; 2)

(?P<GROUPS_SEGMENT>GROUPS_SEGMENT; BEGIN; COMPOSITE;BINARY;\d+
.+


GROUPS_SEGMENT; END)

(?P<MARKS_SEGMENT>MARKS_SEGMENT; BEGIN; COMPOSITE;BINARY;\d+
.+


MARKS_SEGMENT; END)


MARKS_GROUPS_DATA; END''', re.DOTALL
)
